# This is an R script that runs RRAWFLOW from the R prompt
# Run this script after PEST calibration to get final plots and summary output

# Make the functions active
source("rrawflow.r")
source("rf_utilities.r")

# Load input files
vars=loadHydroVars("data/rf_vars.csv")
input=loadHydroData("data/rf_input.csv")
cntrl_pts=loadControl("data/control_pts.csv")
irf=loadIRF("data/rf_irf.csv")

# Run the model and plot png files
output=rrawflow(vars,input,cntrl_pts)
plotHydro(input,output)
plotIRFdays(output)
plotIRFyears(output)

# Write output and plot to R console
write.csv(output,file="rf_output.csv",row.names=F)
write.table(output,file="rf_output.txt",row.names=F,sep="                    ")

# Assess model fit
ylen=length(input$response)
j=1 
yObs=0 # initialize
yOut=0 # initialize
yObsFin=is.finite(input$response) # determine if observed response has finite value
for (i in 1:ylen) {
	if (yObsFin[i]=="TRUE") {# make a record of only finite values for calibration
	yObs[j]=input$response[i]
	yOut[j]=output$response[i]
		if (input$decYear[i]<vars$valStart) {
      calEnd=j
		}	
	j=j+1 
} # end if
} # end for
Obslen=length(yObs)
ssrC=sum((yObs[1:calEnd]-yOut[1:calEnd])^2) # sum of the squared residuals.
ssrV=sum((yObs[(calEnd+1):Obslen]-yOut[(calEnd+1):Obslen])^2) # sum of the squared residuals.
ssdm=sum((yObs-mean(yObs))^2)  # sum of the squared deviations 
							# from the mean for observed response.
Ecal=1-ssrC/ssdm/(calEnd/Obslen) # Nash Sutcliffe coefficient of efficiency for calibration period
Eval=1-ssrV/ssdm/((Obslen-calEnd)/Obslen) # Nash Sutcliffe coefficient of efficiency for validation period

# Calculate system memory for wet and dry period IRFs
areaW=sum(output$hWet) # Curve area for wet period IRF
areaD=sum(output$hDry) # Curve area for dry period IRF
wetFrct=output$hWet/areaW # Sum of wetFrct = 1
dryFrct=output$hDry/areaD # Sum of dryFrct = 1
sumW=0
sumD=0
memW=0
memD=0
if (areaD>0) { 
  for (i in 1:ylen) {
	  sumD=dryFrct[i]+sumD
	  if (sumD<=0.95) { # day on which 95% of curve area is to the left
		  memD=i  # Sytem memory in number of time steps (e.g., days)
	  }
  }
}
if (areaW>0) { 
	for (i in 1:ylen) {
		sumW=wetFrct[i]+sumW
		if (sumW<=0.95) { # day on which 95% of curve area is to the left
			memW=i  # Sytem memory in number of time steps (e.g., days)
		}
	}
} else {
	memW=memD # Wet and dry periods are not distinguished 
}
# Write the summary table
# Note: system memory is not accurate if the curve extends beyond record length (ylen)
smry = list(Ecal=Ecal,Eval=Eval,memoryDry=memD,memoryWet=memW)
write.csv(smry,file="rf_summary.csv",row.names=F)
print("Ecal")
print(Ecal)
print("Eval")
print(Eval)
rm(i,j)

